@@ -34,6 +34,7 @@ import time
 import socket
 import shutil
 import shlex
+from typing import (Optional, Union, Tuple)
 
 import adtlog
 
@@ -229,9 +230,35 @@ def get_unix_socket(path):
     return s
 
 
-def expect(sock, search_bytes, timeout_sec, description=None, echo=False):
-    adtlog.debug('expect: "%s"' % (search_bytes or b'<none>').decode())
-    what = '"%s"' % (description or search_bytes or 'data')
+def matches_expectation(
+    haystack: bytes,
+    needle: Union[None, bytes, str, Tuple[Union[bytes, str]]],
+) -> bool:
+    if needle is None:
+        return True
+    elif isinstance(needle, tuple):
+        for n in needle:
+            if matches_expectation(haystack, n):
+                return True
+        else:
+            return False
+    elif isinstance(needle, bytes):
+        return needle in haystack
+    elif isinstance(needle, str):
+        return needle.encode('utf-8') in haystack
+    else:
+        raise TypeError(repr(needle))
+
+
+def expect(
+    sock: socket.socket,
+    search: Union[None, bytes, str, Tuple[Union[bytes, str]]],
+    timeout_sec: int,
+    description: Optional[str] = None,
+    echo: bool = False,
+):
+    adtlog.debug('expect: %r' % (search,))
+    what = repr(description or search or 'data')
     out = b''
     with timeout(timeout_sec,
                  description and ('timed out waiting for %s' % what) or None):
@@ -243,7 +270,8 @@ def expect(sock, search_bytes, timeout_sec, description=None, echo=False):
             if echo:
                 sys.stderr.buffer.write(block)
             out += block
-            if search_bytes is None or search_bytes in out:
+
+            if matches_expectation(out, search):
                 adtlog.debug('expect: found "%s"' % what)
                 break
 
@@ -333,6 +361,7 @@ def reboot_testbed():
 def cmd_reboot(c, ce):
     global downtmp
     cmdnumargs(c, ce, 0, 1)
+    wait_reboot_args = {}
     if not downtmp:
         bomb("`reboot' when not open")
     if 'reboot' not in caller.hook_capabilities():
@@ -345,12 +374,12 @@ def cmd_reboot(c, ce):
                 '  tar --warning=none --create --absolute-names '
                 '''    -f $d/autopkgtest-tmpdir.tar '%s'; '''
                 '  rm -f /run/autopkgtest-reboot-prepare-mark; '
-                '  exit 0; fi; done; exit 1''' % (directories, downtmp)],
+                '  exit 0; fi; done; exit 1' % (directories, downtmp)],
                downp=True, timeout=copy_timeout)
     adtlog.debug('cmd_reboot: saved current downtmp, rebooting')
 
     try:
-        caller.hook_prepare_reboot()
+        wait_reboot_args = caller.hook_prepare_reboot() or {}
     except AttributeError:
         pass
 
@@ -360,7 +389,7 @@ def cmd_reboot(c, ce):
     else:
         reboot_testbed()
 
-    caller.hook_wait_reboot()
+    caller.hook_wait_reboot(**wait_reboot_args)
 
     # restore downtmp
     check_exec(['sh', '-ec', 'for d in %s; do '
