@@ -224,7 +224,6 @@ class Testbed:
         self.scratch = pl[0]
         self.deps_installed = []
         self.apt_pin_for_releases = []
-        self.recommends_installed = False
         self.exec_cmd = list(map(urllib.parse.unquote, self.command('print-execute-command', (), 1)[0].split(',')))
         self.caps = self.command('capabilities', (), None)
         if self.needs_internet in ['try', 'run']:
@@ -248,13 +247,21 @@ class Testbed:
         adtlog.info('testbed dpkg architecture: ' + self.dpkg_arch)
 
         # do we have eatmydata?
-        (code, out, err) = self.execute(['which', 'eatmydata'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        (code, out, err) = self.execute(
+            ['sh', '-ec', 'command -v eatmydata'],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        )
         if code == 0:
             adtlog.debug('testbed has eatmydata')
             self.eatmydata_prefix = [out.strip()]
 
         # record package versions of pristine testbed
-        if self.output_dir and self.execute(['which', 'dpkg-query'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)[0] == 0:
+        if self.output_dir and self.execute(
+            ['sh', '-ec', 'command -v dpkg-query'],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        )[0] == 0:
             pkglist = TempPath(self, 'testbed-packages', autoclean=False)
             self.check_exec(['sh', '-ec', "dpkg-query --show -f '${Package}\\t${Version}\\n' > %s" % pkglist.tb])
             pkglist.copyup()
@@ -391,29 +398,27 @@ class Testbed:
                 adtlog.info('rebooting testbed after setup commands that affected boot')
                 self.reboot()
 
-    def reset(self, deps_new, with_recommends):
+    def reset(self, deps_new):
         '''Reset the testbed, if possible and necessary'''
 
-        adtlog.debug('testbed reset: modified=%s, deps_installed=%s(r: %s), deps_new=%s(r: %s)' %
-                     (self.modified, self.deps_installed, self.recommends_installed,
-                      deps_new, with_recommends))
+        adtlog.debug('testbed reset: modified=%s, deps_installed=%s, deps_new=%s' %
+                     (self.modified, self.deps_installed, deps_new))
         if 'revert' in self.caps and (
-                self.modified or self.recommends_installed != with_recommends or
+                self.modified or
                 [d for d in self.deps_installed if d not in deps_new]):
             adtlog.debug('testbed reset')
             pl = self.command('revert', (), 1)
             self._opened(pl)
         self.modified = False
 
-    def install_deps(self, deps_new, recommends, shell_on_failure=False, synth_deps=[]):
+    def install_deps(self, deps_new, shell_on_failure=False, synth_deps=[]):
         '''Install dependencies into testbed'''
-        adtlog.debug('install_deps: deps_new=%s, recommends=%s' % (deps_new, recommends))
+        adtlog.debug('install_deps: deps_new=%s' % deps_new)
 
         self.deps_installed = deps_new
-        self.recommends_installed = recommends
         if not deps_new:
             return
-        self.satisfy_dependencies_string(', '.join(deps_new), 'install-deps', recommends, shell_on_failure=shell_on_failure, synth_deps=synth_deps)
+        self.satisfy_dependencies_string(', '.join(deps_new), 'install-deps', shell_on_failure=shell_on_failure, synth_deps=synth_deps)
 
     def needs_reset(self):
         # show what caused a reset
@@ -599,17 +604,17 @@ class Testbed:
                     return True
         return False
 
-    def _run_apt_install(self, what, prefix, recommends, ignorerc=False):
+    def _run_apt_install(self, what, prefix, ignorerc=False):
         '''actually run apt-get install'''
 
         # capture status-fd to stderr
         (rc, _, serr) = self.execute(['/bin/sh', '-ec', '%s apt-get install '
                                       '--assume-yes %s '
                                       '-o APT::Status-Fd=3 '
-                                      '-o APT::Install-Recommends=%s '
+                                      '-o APT::Install-Recommends=false '
                                       '-o Dpkg::Options::=--force-confnew '
                                       '-o Debug::pkgProblemResolver=true 3>&2 2>&1' %
-                                      (prefix, what, recommends)],
+                                      (prefix, what)],
                                      kind='install', stderr=subprocess.PIPE)
         if not ignorerc and rc != 0:
             adtlog.debug('apt-get install %s failed; status-fd:\n%s' % (what, serr))
@@ -620,7 +625,7 @@ class Testbed:
             else:
                 raise adtlog.AptPermanentError
 
-    def install_apt(self, deps, recommends=False, shell_on_failure=False, synth_deps=[]):
+    def install_apt(self, deps, shell_on_failure=False, synth_deps=[]):
         '''Install dependencies with apt-get into testbed
 
         This requires root privileges and a writable file system.
@@ -643,7 +648,7 @@ Description: satisfy autopkgtest test dependencies
 ''' % (self.dpkg_arch, deps))
 
         deb = TempPath(self, 'autopkgtest-satdep.deb')
-        subprocess.check_call(['dpkg-deb', '-b', pkgdir, deb.host],
+        subprocess.check_call(['dpkg-deb', '-Zxz', '-b', pkgdir, deb.host],
                               stdout=subprocess.PIPE)
         shutil.rmtree(pkgdir)
         deb.copydown()
@@ -661,7 +666,7 @@ Description: satisfy autopkgtest test dependencies
 
             rc = 0
             try:
-                self._run_apt_install('--fix-broken', ' '.join(self.eatmydata_prefix), recommends)
+                self._run_apt_install('--fix-broken', ' '.join(self.eatmydata_prefix))
                 need_explicit_install = []
                 for dep in synth_deps:
                     if dep != list(dep):
@@ -693,7 +698,7 @@ Description: satisfy autopkgtest test dependencies
 
                 if need_explicit_install:
                     adtlog.debug('installing real packages of test dependencies: %s' % need_explicit_install)
-                    self._run_apt_install(' '.join(need_explicit_install), ' '.join(self.eatmydata_prefix), recommends)
+                    self._run_apt_install(' '.join(need_explicit_install), ' '.join(self.eatmydata_prefix))
 
             # check if apt failed during package download, which might be a
             # transient error, so retry
@@ -732,7 +737,7 @@ Description: satisfy autopkgtest test dependencies
                                'data about failing dependencies in test logs')
                 self._run_apt_install('--simulate ' + deps.replace(',', ''),
                                       ' '.join(self.eatmydata_prefix),
-                                      recommends, ignorerc=True)
+                                      ignorerc=True)
 
                 if shell_on_failure:
                     self.run_shell()
@@ -869,7 +874,7 @@ Description: satisfy autopkgtest test dependencies
         if self.execute(['sh', adtlog.verbosity >= 2 and '-exc' or '-ec', script], kind='install')[0] != 0:
             self.bomb('Failed to update click AppArmor rules')
 
-    def satisfy_dependencies_string(self, deps, what, recommends=False,
+    def satisfy_dependencies_string(self, deps, what,
                                     build_dep=False, shell_on_failure=False, synth_deps=[]):
         '''Install dependencies from a string into the testbed'''
 
@@ -912,10 +917,10 @@ Description: satisfy autopkgtest test dependencies
         adtlog.debug('can use apt-get on testbed: %s' % can_apt_get)
 
         if can_apt_get:
-            self.install_apt(deps, recommends, shell_on_failure, synth_deps)
+            self.install_apt(deps, shell_on_failure, synth_deps)
         else:
             has_dpkg_checkbuilddeps = self.execute(
-                ['which', 'dpkg-checkbuilddeps'],
+                ['sh', '-ec', 'command -v dpkg-checkbuilddeps'],
                 stdout=subprocess.DEVNULL,
                 stderr=subprocess.DEVNULL
             )[0] == 0
@@ -956,7 +961,11 @@ Description: satisfy autopkgtest test dependencies
         need_click_restore = self.apparmor_click(test.clicks, test.installed_clicks)
 
         # record installed package versions
-        if self.output_dir and self.execute(['which', 'dpkg-query'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)[0] == 0:
+        if self.output_dir and self.execute(
+            ['sh', '-ec', 'command -v dpkg-query'],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        )[0] == 0:
             pkglist = TempPath(self, test.name + '-packages.all', autoclean=False)
             self.check_exec([
                 'sh', '-ec', "dpkg-query --show -f '${Package}\\t${Version}\\n' > %s" % pkglist.tb])
