@@ -251,11 +251,11 @@ for details.'''
     g_setup.add_argument('-U', '--apt-upgrade', dest='setup_commands',
                          action='append_const',
                          const='''(O=$(bash -o pipefail -ec 'apt-get update | tee /proc/self/fd/2') ||'''
-                         '{ [ "${O%404*Not Found*}" = "$O" ] || exit 100; sleep 15; apt-get update; }'''
+                         '{ [ "${O%404*Not Found*}" = "$O" ] || exit 100; sleep 15; apt-get update; }'
                          ' || { sleep 60; apt-get update; } || false)'
-                         ' && $(which eatmydata || true) apt-get dist-upgrade -y -o '
+                         ' && $(command -v eatmydata || true) apt-get dist-upgrade -y -o '
                          'Dpkg::Options::="--force-confnew"'
-                         ' && $(which eatmydata || true) apt-get --purge autoremove -y',
+                         ' && $(command -v eatmydata || true) apt-get --purge autoremove -y',
                          help='Run apt update/dist-upgrade before the tests')
     g_setup.add_argument('--setup-commands-boot', metavar='COMMANDS_OR_PATH',
                          action='append', default=[],
