@@ -270,6 +270,36 @@ def _debian_packages_from_source(srcdir):
     return (packages, packages_no_arch)
 
 
+def _debian_recommends_from_source(srcdir, testbed_arch):
+    deps = ''
+    for st in parse_rfc822(os.path.join(srcdir, 'debian/control')):
+        if 'Recommends' in st:
+            deps += st['Recommends'] + ','
+
+    # resolve arch specific dependencies and build profiles
+    perl = subprocess.Popen(['perl', '-'], stdin=subprocess.PIPE,
+                            stdout=subprocess.PIPE)
+    code = '''use Dpkg::Deps;
+              $dep = deps_parse('%s', reduce_arch => 1,
+                                host_arch => '%s');
+              $out = $dep->output();
+              print $out, "\\n";
+              ''' % (deps, testbed_arch)
+    deps = perl.communicate(code.encode('UTF-8'))[0].decode('UTF-8').strip()
+    if perl.returncode != 0:
+        raise InvalidControl('source', 'Invalid Recommends')
+
+    deps = [d.strip() for d in deps.split(',')]
+
+    # The following check is a bit silly as there were apparently no
+    # recommeds, but still the test asked for it. Let's not fail on
+    # that, but also let's not add the empty string.
+    if deps == ['']:
+        deps = []
+
+    return deps
+
+
 def _debian_build_deps_from_source(srcdir, testbed_arch):
     deps = ''
     for st in parse_rfc822(os.path.join(srcdir, 'debian/control')):
@@ -365,8 +395,8 @@ def _synthesize_deps(dep, testbed_arch):
 def _parse_debian_depends(testname, dep_str, srcdir, testbed_arch):
     '''Parse Depends: line in a Debian package
 
-    Split dependencies (comma separated), validate their syntax, and expand @
-    and @builddeps@. Return a list of dependencies.
+    Split dependencies (comma separated), validate their syntax, and expand @,
+    @builddeps@ and @recommends@. Return a list of dependencies.
 
     This may raise an InvalidControl exception if there are invalid
     dependencies.
@@ -391,6 +421,10 @@ def _parse_debian_depends(testname, dep_str, srcdir, testbed_arch):
             for d in _debian_build_deps_from_source(srcdir, testbed_arch):
                 adtlog.debug('synthesised dependency %s' % d)
                 deps.append(d)
+        elif alt_group_str == '@recommends@':
+            for d in _debian_recommends_from_source(srcdir, testbed_arch):
+                adtlog.debug('synthesised dependency %s' % d)
+                deps.append(d)
         else:
             synthdep_alternatives = []
             for dep in alt_group_str.split('|'):
@@ -550,6 +584,11 @@ def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
             restrictions = record.get('Restrictions', '').replace(
                 ',', ' ').split()
 
+            # needs-recommends is deprecated, but until removed, let's
+            # replace it with @recommends@
+            if 'needs-recommends' in restrictions:
+                record['Depends'] = record.get('Depends', '@') + ',@recommends@'
+
             feature_test_name = None
             features = []
             record_features = record.get('Features', '').replace(
